<?php
/*
Plugin Name: Redirect by Custom Field
Description: Changes the URLs pointing to pages and posts which have a <code>redirect</code> custom field, using that entry as the URL instead.
Author: mitcho (Michael 芳貴 Erlewine)
Version: 0.6
Author URI: http://mitcho.com/
Donate link: http://tinyurl.com/donatetomitcho
*/

// This section will redirect URL's when the real permalink is hit directly.
// Maybe I should have used the rewrite API or the canonical_redirect filter instead?
// Note: I didn't use rewrite because I didn't want to have to keep track of all redirect mappings.
add_action('parse_query', 'redirect_header', 10, 1);
function redirect_header($query) {
	if (is_singular() && isset($query->queried_object)) {
		$link = redirect_by_custom_field(false, $query->queried_object);
		if ($link !== false) {
			wp_redirect(esc_url($link), 301);
			exit;
		}
	}
	return $query;
}

// This section will replace every instance of the permalink with the redirect URL.
add_filter('page_link', 'redirect_by_custom_field', 10, 2);
add_filter('post_link', 'redirect_by_custom_field', 10, 2);
function redirect_by_custom_field($link, $postarg = null) {
	global $post;
	if (is_admin())
		return $link;
	
	if (is_object($postarg))
		$id = $postarg->ID;
	else if (is_int($postarg))
		$id = $postarg;
	else if (is_object($post))
		$id = $post->ID;
	else
		return $link;
	
	$redirect = get_redirect_url($id);
	if ($redirect)
		return $redirect;
	return $link;
}

// id must be int
function get_redirect_url($id) {
	$redirect = get_post_meta($id, 'redirect', true);
	if ($redirect) {
		if (is_array($redirect))
			$redirect = $redirect[0];
		return $redirect;
	}
	return false;
}

add_filter('get_sample_permalink_html', 'redirect_display_modifier', 10, 4);
function redirect_display_modifier($return, $id, $new_title, $new_slug) {
	$redirect = get_redirect_url($id);
	if ($redirect)
		$return = "<strong>" . __("Redirect:", 'redirect-by-custom-field') . "</strong> " . esc_html($redirect) . "<style>#titlediv {margin-bottom: 30px;}</style><br/>" . $return;
	return $return;
}
